@@ -862,52 +862,112 @@ NumericVector cpp_diag_XUtX(NumericMatrix X, NumericMatrix U){
 }
 
 
+class simple_vec_double{
+    simple_vec_double() = delete;
+    double *px_double = nullptr;
+    int *px_int = nullptr;
+    int n;
+    bool is_real;
+public:
+    simple_vec_double(SEXP x);
+    double operator[](int);
+};
+
+simple_vec_double::simple_vec_double(SEXP x){
+
+    n = Rf_length(x);
+
+    if(TYPEOF(x) == REALSXP){
+        px_double = REAL(x);
+        is_real = true;
+
+    } else if(TYPEOF(x) == INTSXP){
+        px_int = INTEGER(x);
+        is_real = false;
+
+    } else {
+        stop("Error: Wrong argument type in cpp_factor_matrix.");
+    }
+}
+
+double simple_vec_double::operator[](int i){
+    if(i >= n){
+        return 1;
+    } else if(is_real){
+        return px_double[i];
+    } else {
+        return static_cast<double>(px_int[i]);
+    }
+}
+
+
 // [[Rcpp::export]]
-NumericVector cpp_factor_matrix(IntegerVector fact, bool any_na){
+NumericVector cpp_factor_matrix(IntegerVector fact, LogicalVector is_na_all, IntegerVector who_is_dropped, SEXP var, CharacterVector col_names){
     // fact: integer vector from 1 (!) to K, can contain NAs
     // Checking Na is cheap as opposed to populating the matrix, but having an argument avoids creating a new object
 
     int n = fact.length();
     int K = 0;
-    LogicalVector is_na(any_na ? n : 1, false);
 
-    // Finding out the NAs and the nber of cols
-    if(any_na){
-        for(int i=0 ; i<n ; ++i){
-            if(IntegerVector::is_na(fact[i])){
-                is_na[i] = true;
-            } else if(K < fact[i]){
-                K = fact[i];
-            }
+    // Finding out the TOTAL nber of cols (before removal)
+    for(int i=0 ; i<n ; ++i){
+        if(!is_na_all[i] && K < fact[i]){
+            K = fact[i];
         }
-    } else {
-        for(int i=0 ; i<n ; ++i){
-            if(K < fact[i]){
-                K = fact[i];
+    }
+
+    // Are there values to remove? If so, we create a mapping
+    int n_drop = who_is_dropped[0] == -1 ? 0 : Rf_length(who_is_dropped);
+    bool IS_REMOVAL = n_drop > 0;
+    std::vector<int> mapping;
+
+    if(IS_REMOVAL){
+        mapping.resize(K);
+
+        for(int k=0 ; k<K ; ++k){
+            mapping[k] = k;
+        }
+
+        // In mapping: values equal to -1 mean dropped value
+
+        int k_drop = 0;
+        for(int k=0 ; k<K ; ++k){
+            if(k_drop < n_drop && k + 1 == who_is_dropped[k_drop]){
+                ++k_drop;
+                mapping[k] = -1;
+            } else {
+                mapping[k] -= k_drop;
             }
         }
+
+        K -= k_drop;
     }
 
     NumericMatrix res(n, K);
 
+    // The interacted var
+    simple_vec_double my_var(var);
+
     // Filling the matrix
-    if(any_na){
-        for(int i=0 ; i<n ; ++i){
-            if(IntegerVector::is_na(fact[i])){
-                // we fill the row
-                for(int k=0 ; k<K ; ++k){
-                    res(i, k) = NA_REAL;
-                }
-            } else {
-                res(i, fact[i] - 1) = 1;
+    for(int i=0 ; i<n ; ++i){
+        if(is_na_all[i]){
+            // we fill the row
+            for(int k=0 ; k<K ; ++k){
+                res(i, k) = NA_REAL;
             }
-        }
-    } else {
-        for(int i=0 ; i<n ; ++i){
-            res(i, fact[i] - 1) = 1;
+
+        } else if(IS_REMOVAL) {
+            if(mapping[fact[i] - 1] != -1){
+                res(i, mapping[fact[i] - 1]) = my_var[i];
+            }
+
+        } else {
+            res(i, fact[i] - 1) = my_var[i];
         }
     }
 
+    colnames(res) = col_names;
+
     return res;
 }
 
